
/******************************************************************************
* MODULE     : defaults.hpp
* DESCRIPTION: Templates and default implementations of several operations
* COPYRIGHT  : (C) 2003  Joris van der Hoeven
*******************************************************************************
* This software falls under the GNU general public license and comes WITHOUT
* ANY WARRANTY WHATSOEVER. See the file $TEXMACS_PATH/LICENSE for more details.
* If you don't have this file, write to the Free Software Foundation, Inc.,
* 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
******************************************************************************/

#ifndef __MML_DEFAULTS_HPP
#define __MML_DEFAULTS_HPP
#include "base.hpp"
namespace mml {

/******************************************************************************
* Printing
******************************************************************************/

template<typename C> std::ostream&
print_lisp (std::ostream& out, C x) {
  return out << x;
}

template<typename T> std::ostream&
print_lisp (std::ostream& out, char* fun, const T& x) {
  out << "(" << fun << " ";
  print_lisp (out, x);
  return out << ")";
}

template<typename T, typename U> std::ostream&
print_lisp (std::ostream& out, char* fun, const T& x, const U& y) {
  out << "(" << fun << " ";
  print_lisp (out, x);
  out << " ";
  print_lisp (out, y);
  return out << ")";
}

/******************************************************************************
* Hashing
******************************************************************************/

inline nat hash (int c) { return (nat) c; }
inline nat hash (nat c) { return (nat) c; }
inline nat hash (char c) { return (nat) c; }
inline nat hash (unsigned char c) { return (nat) c; }

/******************************************************************************
* Arithmetic
******************************************************************************/

template<typename C, typename D> C&
operator += (C& x, const D& y) {  return x= x+y; }
template<typename C, typename D> C&
operator -= (C& x, const D& y) { return x= x-y; }
template<typename C, typename D> C&
operator *= (C& x, const D& y) { return x= x*y; }
template<typename C, typename D> C&
operator /= (C& x, const D& y) { return x= x/y; }
template<typename C, typename D> C&
operator %= (C& x, const D& y) { return x= x%y; }

template<typename C, typename D> C&
operator <<= (C& x, D y) { return x= x<<y; }
template<typename C, typename D> C&
operator >>= (C& x, D y) { return x= x>>y; }

/******************************************************************************
* Comparison
******************************************************************************/

template<typename C> int
sign (const C& x) {
  if (x>0) return 1;
  if (x<0) return -1;
  return 0;
}

template<typename C> int
compare (const C& x, const C& y) {
  return sign (x-y);
}

template<typename C> C min (const C& x, const C& y) { return x<y? x: y; }
template<typename C> C max (const C& x, const C& y) { return x>y? x: y; }

/******************************************************************************
* Special functions
******************************************************************************/

template<typename C> C sqrt (const C& x) { assert (x == C(1)); return 1; }
template<typename C, typename D> C
pow (const C& x, const D& y) { assert (x == C(1)); return 1; }

template<typename C> C exp (const C& x) { assert (x == C(0)); return 1; }
template<typename C> C log (const C& x) { assert (x == C(1)); return 0; }
template<typename C> C sin (const C& x) { assert (x == C(0)); return 0; }
template<typename C> C cos (const C& x) { assert (x == C(0)); return 1; }
template<typename C> C tan (const C& x) { assert (x == C(0)); return 0; }

} // namespace mml
#endif // __MML_DEFAULTS_HPP
