
/******************************************************************************
* MODULE     : jet.hpp
* DESCRIPTION: low level operations on polynomial or serial segments
* COPYRIGHT  : (C) 2003  Joris van der Hoeven
*******************************************************************************
* This software falls under the GNU general public license and comes WITHOUT
* ANY WARRANTY WHATSOEVER. See the file $TEXMACS_PATH/LICENSE for more details.
* If you don't have this file, write to the Free Software Foundation, Inc.,
* 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
******************************************************************************/

#ifndef __MML__JET__HPP
#define __MML__JET__HPP
#include "segment.hpp"

namespace mml {
#define TMPL template<class C>

/******************************************************************************
* Vectorial operations
******************************************************************************/

#define jet_set segment_set
#define jet_clear segment_clear
#define jet_copy segment_copy
#define jet_add segment_add
#define jet_sub segment_sub
#define jet_neg segment_neg

TMPL void jet_mul (C* dest, C* s, C c, nat n) { segment_mul (dest, s, c, n); }
TMPL void jet_mul (C* dest, C c, nat n) { segment_mul (dest, c, n); }
TMPL void jet_div (C* dest, C* s, C c, nat n) { segment_div (dest, s, c, n); }
TMPL void jet_div (C* dest, C c, nat n) { segment_div (dest, c, n); }

TMPL int
jet_sign (C* s, nat n) {
  while (n != 0) {
    int sgn= sign (*s);
    if (sgn != 0) return sgn;
    s++; n--;
  }
  return 0;
}

TMPL int
jet_cmp (C* s1, C* s2, nat n) {
  while (n != 0) {
    int sgn= compare (*s1, *s2);
    if (sgn != 0) return sgn;
    s1++; s2++; n--;
  }
  return 0;
}

/******************************************************************************
* Multiplication
******************************************************************************/

TMPL void
jet_mul (C* dest, C* s1, C* s2, nat n1, nat n2) {
  segment_set (dest, 0, n1+n2-1);
  while (n2 != 0) {
    segment_mul_add (dest, s1, *s2, n1);
    dest++; s2++; n2--;
  }
}

#undef TMPL
} // namespace mml
#endif //__MML__JET__HPP
